from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n


import commands
import re
import os
import time
import Queue
import urllib



#
# Sensor for retrieving weather information.
#
class WeatherSensor(Sensor):

    WEATHER_SOURCE = "http://weather.yahoo.com/search/" \
                     "weather?p=%(webcity)s,%(webcountry)s"

    # regular expressions to parse the weather data
    RE_TEMPERATURE = re.compile("\n(?P<temp>\d+)")
    RE_SKY = re.compile("--\>\n(?P<sky>[^\<]+)\</font\>\</td\>")
    RE_ICON = re.compile("\<img \nsrc=\"http://us.i1.yimg.com/us.yimg.com/"
                         "i/us/we/52/(?P<icon>\d+).gif\"\n")

    CENTIGRADE = "centigrade"
    FAHRENHEIT = "fahrenheit"


    WEATHER_ICONS = {1: "rain",
                     2: "rain",
                     3: "thunder",
                     4: "thunder",
                     5: "rain",
                     6: "rain",
                     7: "ice",
                     8: "rain",
                     9: "drizzle",
                     10: "rain",
                     11: "rain",
                     12: "rain",
                     13: "snow",
                     14: "snow",
                     15: "snow",
                     16: "snow",
                     17: "thunder",
                     18: "rain",
                     19: "fog",
                     20: "fog",
                     21: "fog",
                     22: "fog",
                     23: "wind",
                     24: "wind",
                     25: "ice",
                     26: "cloud",
                     27: "luna-nori", # moon
                     28: "mostly-cloudy",
                     29: "luna-ceatza", # moon
                     30: "suncloud",
                     31: "moon",
                     32: "sun",
                     33: "moon",
                     34: "sun-light-clouds",
                     35: "thunder",
                     36: "sun",
                     37: "thunder",
                     38: "thunder",
                     39: "rain",
                     40: "rain",
                     41: "snow",
                     42: "snow",
                     43: "snow",
                     44: "suncloud",
                     45: "moon-rain", # moon
                     46: "moon-snow", # moon
                     47: "moon-tstorm" #moon
                     }



    def __init__(self):

        global _; _ = i18n.Translator("weather-sensor")
        
        self.__queue = Queue.Queue(1)
        self.__path = os.getcwd()


        Sensor.__init__(self)

        self._set_config_type("city", TYPE_STRING, "Tokyo")
        self._set_config_type("country", TYPE_STRING, "Japan")
        self._set_config_type("units", TYPE_STRING, self.CENTIGRADE)
        self._set_config_type("t_update", TYPE_STRING, "30")

        self._add_thread(self.__get_weather_thread)
        self._add_timer(0, self.__get_weather)



    #
    # Returns the configuration GUI.
    #
    def get_configurator(self):

        configurator = self._new_configurator()
        configurator.set_name(_("Weather"))
        configurator.add_title(_("Server Settings"))
        configurator.add_entry(_("City:"), "city", _("The (English) name "
                                                     "of your city"))
        configurator.add_entry(_("Country:"), "country", _("The (English) "
                                                           "name of your "
                                                           "country"))
        configurator.add_entry(_("Update Time (Mins):"), "t_update", _("The "
                                                    "amount of time to update "
                                                    "the display in minutes"))
        configurator.add_title(_("Measurement"))
        configurator.add_option(_("Measurement unit:"), "units",
                                _("One of Fahrenheit or Centigrade (Celsius)"),
                                [(_("Centigrade"), self.CENTIGRADE),
                                 (_("Fahrenheit"), self.FAHRENHEIT)])
        
        return configurator



    def __fahrenheit2centigrade(self, value):

        return (value - 32) * (5 / 9.0)



    #
    # Retrieves the weather data.
    #
    def __get_weather(self):

        # check the queue regularly
        if (not self.__queue.empty()):
            output = self.__queue.get()
            self._send_output(output)

        self._add_timer(500, self.__get_weather)
        



    #
    # Thread function for retrieving the weather data since this may take time
    # and must not block the application.
    #
    def __get_weather_thread(self):

        counter = 0
        old_values = ("", "", "", "")
        current_time = int(time.time())
        # Set us back t_update seconds so the if statment will be true
        t_update = int(self._get_config("t_update"))
        last_update = current_time - (t_update * 60) - 1
        
        while (1):
            if (self._is_stopped()): break

            city = self._get_config("city")
            country = self._get_config("country")
            units = self._get_config("units")
            t_update = int(self._get_config("t_update"))
            webcity = urllib.quote(city)
            webcountry = urllib.quote(country)

            # Initalize the current time. 
            current_time = int(time.time())


            if ((city, country, units, t_update) != old_values or \
                last_update + (t_update * 60) <= current_time):
                # At least 300/60 = 5 minutes this will run. Or so I thought...
                # Sleep can take a few seconds to many minutes, it's not
                # accurate at all.  This is a problem for me, I'm picky.  CN
                old_values = (city, country, units, t_update)
                current_time = int(time.time())
                last_update = int(time.time())

                if (units == self.CENTIGRADE):
                    measuring = "&#176;C"
                else:
                    measuring = "&#176;F"

                fh = urllib.urlopen(self.WEATHER_SOURCE % vars())
                data = fh.read()

                output = self._new_output()
                output.set("location", city)
            
                try:
                    temperature = \
                          float(self.RE_TEMPERATURE.search(data).group("temp"))
                    if (units == self.CENTIGRADE):
                        temperature = self.__fahrenheit2centigrade(temperature)
						
                    
                    sky = self.RE_SKY.search(data).group("sky")
                    icon_nr = self.RE_ICON.search(data).group("icon")
                    icon = self.WEATHER_ICONS.get(int(icon_nr), 30)
                
                    output.set("temperature", int(temperature))
                    output.set("sky", sky)
                    output.set("icon", os.path.join(self.__path, "icons",
                                                "%(icon)s.png" % vars()))

                except:
                    output.set("temperature", "Retrieval Failed")
                    output.set("sky", "N/A")
                    output.set("icon", os.path.join(self.__path,
                                                    "icons/unknown.png"))
                    measuring = ''
                output.set("unit", measuring)
                self.__queue.put(output)
            #end if
            time.sleep(1)
        #end while
            


print """
The weather sensor uses data from weather.yahoo.com.
Yahoo! claims:

************************** NOTICE *******************************
The data (conditions, forecasts, news, images, logos) contained in
this page are copyrighted by Yahoo! Inc. and the Weather Channel
Enterprises, Inc. You are prohibited from using or repurposing this
data in any way without express written consent from Yahoo! Inc. and
the Weather Channel Enterprises, Inc.
If you are looking for a source of weather data, please see the
National Weather Service Website at http://www.nws.noaa.gov/. Their
data is public information, to be used with appropriate
byline/photo/image credits.
************************** NOTICE *******************************
"""


def new_sensor(args): return apply(WeatherSensor, args)
